﻿

Imports System
Imports System.IO.Ports
Imports System.Threading

Public Class Form1
    Delegate Sub UpdatePB(ByVal val As Byte)
    Delegate Sub UpdatePD(ByVal val As Byte)
    Public UpdatePortBDelegate As UpdatePB
    Public UpdatePortDDelegate As UpdatePD

    Const BitNA As Integer = 0
    Const BitOff As Integer = 1
    Const BitOn As Integer = 2

    Dim PortBValue As Byte
    Dim PortDValue As Integer
    Dim PortDEnable As Integer
    Dim ThreadContinue As Boolean

    '
    ' フォーム初期化する
    '
    Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。

        UpdatePortBDelegate = New UpdatePB(AddressOf UpdatePortB)
        UpdatePortDDelegate = New UpdatePD(AddressOf UpdatePortD)
        PortBValue = &H33                   ' PORT Bの値を初期化する
        PortDEnable = &H7C                  ' PORT DのENABLEを初期化する
        PortDValue = &H4A                   ' PORT Dの値を初期化する

        PortList.Items.Clear()              ' 通信ポート名リストを初期化する
        PortList.Items.Add("Close")         ' 通信ポート名リストの先頭に、"Close"を挿入する
        Dim portName As String              ' 作業の文字列変数を定義する
        For Each portName In SerialPort.GetPortNames() ' SerialPortクラスで、通信ポート名一覧を取得する
            PortList.Items.Add(portName)    ' 通信ポート名の一覧を、通信ポート名リストに転記する
        Next portName
        UpdatePortB(PortBValue)                       ' PORT Bの画面表示を行う
        UpdatePortD(PortDValue)                       ' PORT Dの画面表示を行う

        ThreadContinue = True
        BackgroundWorker1.RunWorkerAsync()
    End Sub

    '
    '   ファームウェアに対して、メッセージを編集してから送信するサブルーチン
    '
    Private Sub WritePort(ByVal cmnd As String, ByVal portName As String, ByVal portVal As Integer)
        Dim valString As String
        If SerialPort1.IsOpen Then      ' 通信ポートがオープンしているかチェックする
            If (cmnd = "W") Then    ' 書込みコマンドなら、'W'にポート名と設定値の16進表示文字（2文字）
                valString = String.Format("{0}{1}{2:X2}", cmnd, portName, portVal And &HFF)
            Else                    ' 読込みコマンドなら、'R'にポート名だけ
                valString = String.Format("{0}{1}", cmnd, portName)
            End If
            Try         ' 通信メッセージを通信ポートに書き込む
                SerialPort1.WriteLine(valString)
            Catch       ' 書込みエラーが発生しても何もしていない。
            End Try
        End If
    End Sub

    '
    '   ポートの値を反転してファームウェアに書込む
    '
    Private Sub InvertBitPortB(ByVal pos As Integer)
        PortBValue = PortBValue Xor (1 << pos)      ' ポートの値を排他的論理和演算で反転する。演算子"<<"は左シフト
        WritePort("W", "B", PortBValue)             ' ファームウェアにポートの値を書込む
    End Sub

    '
    '   PORTのビットパターンを画面表示する
    '
    Private Sub UpdateBit(ByVal bitValue As Integer, ByVal bitEnable As Integer, ByVal bitPos As Integer, ByRef pict As PictureBox, ByVal picBase As Integer)
        If ((bitEnable And (1 << bitPos)) <> 0) Then        ' ENABLEビットがONなら、
            If ((bitValue And (1 << bitPos)) <> 0) Then             ' 当該ビットがONなら、
                pict.Image = ImageList1.Images(BitOn + picBase)     '   指定されたPictureBoxに、オンの画像を設定する
            Else                                                    ' 当該ビットがOFFなら
                pict.Image = ImageList1.Images(BitOff + picBase)    '   指定されたPictureBoxに、オフの画像を設定する
            End If
        Else                                                ' ENABLEビットがOFFなら、
            pict.Image = ImageList1.Images(BitNA + picBase)         '   指定されたPictureBoxに、NotAvailableの画像を設定する
        End If
    End Sub

    '
    '   PORTBの値をを設定して、表示を更新する
    '
    Public Sub UpdatePortB(ByVal val As Byte)
        PortBValue = val
        UpdateBit(val, &HFF, 0, PictureBox1, 0)   ' PB0の表示を更新する
        UpdateBit(val, &HFF, 1, PictureBox2, 0)   ' PB1の表示を更新する
        UpdateBit(val, &HFF, 2, PictureBox3, 0)   ' PB2の表示を更新する
        UpdateBit(val, &HFF, 3, PictureBox4, 0)   ' PB3の表示を更新する
        UpdateBit(val, &HFF, 4, PictureBox5, 0)   ' PB4の表示を更新する
        UpdateBit(val, &HFF, 5, PictureBox6, 0)   ' PB5の表示を更新する
        UpdateBit(val, &HFF, 6, PictureBox7, 0)   ' PB6の表示を更新する
        UpdateBit(val, &HFF, 7, PictureBox8, 0)   ' PB7の表示を更新する
    End Sub

    '
    '   PORTDの値をを設定して、表示を更新する
    '
    Public Sub UpdatePortD(ByVal val As Byte)
        PortDValue = val
        UpdateBit(val, PortDEnable, 0, PictureBox9, 3)    ' PD0の表示を更新する
        UpdateBit(val, PortDEnable, 1, PictureBox10, 3)   ' PD1の表示を更新する
        UpdateBit(val, PortDEnable, 2, PictureBox11, 3)   ' PD2の表示を更新する
        UpdateBit(val, PortDEnable, 3, PictureBox12, 3)   ' PD3の表示を更新する
        UpdateBit(val, PortDEnable, 4, PictureBox13, 3)   ' PD4の表示を更新する
        UpdateBit(val, PortDEnable, 5, PictureBox14, 3)   ' PD5の表示を更新する
        UpdateBit(val, PortDEnable, 6, PictureBox15, 3)   ' PD6の表示を更新する
        UpdateBit(val, PortDEnable, 7, PictureBox16, 3)   ' PD7の表示を更新する
    End Sub

    '
    '   フォームを閉じるときの処理
    '
    Private Sub Form1_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        ThreadContinue = False          'スレッドの無限ループを終了する
        If SerialPort1.IsOpen Then      'もし、通信ポートが開いていたら
            SerialPort1.Close()         '   通信ポートを閉じる
        End If
        Thread.Sleep(10)                'スレッドが確実に終了するのを待つ
    End Sub

    '
    ' リストボックスで、選択されている項目が変更されたときに発生するイベント
    '
    Private Sub PortList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PortList.SelectedIndexChanged
        If SerialPort1.IsOpen Then                          ' 通信ポートがオープンされているとき、
            SerialPort1.Close()                             '   通信ポートをクローズする
        End If

        If (PortList.SelectedIndex > 0) Then                ' 通信ポートが指定されたとき、
            SerialPort1.PortName = PortList.SelectedItem    ' 通信ポート名をSerialPortのポート名に転記する
            Try
                SerialPort1.Open()                          ' 通信ポートをオープンする
                WritePort("R", "B", 0)                      ' ファームウェアに、PORT Bの読込みを指示する。
                Thread.Sleep(1)
                WritePort("R", "D", 0)                      ' ファームウェアに、PORT Dの読込みを指示する。
            Catch                                           ' エラー発生時、
                PortList.SelectedIndex = 0                  ' ポートリストの選択項目をCloseにする
            End Try
        End If
    End Sub

    '
    '       ビット０がクリックされたときのイベント処理
    '
    Private Sub PictureBox1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox1.Click
        InvertBitPortB(0)
        UpdateBit(PortBValue, &HFF, 0, PictureBox1, 0)   ' PB0の表示を更新する
    End Sub

    '
    '       ビット１がクリックされたときのイベント処理
    '
    Private Sub PictureBox2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox2.Click
        InvertBitPortB(1)
        UpdateBit(PortBValue, &HFF, 1, PictureBox2, 0)   ' PB1の表示を更新する
    End Sub

    '
    '       ビット２がクリックされたときのイベント処理
    '
    Private Sub PictureBox3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox3.Click
        InvertBitPortB(2)
        UpdateBit(PortBValue, &HFF, 2, PictureBox3, 0)   ' PB2の表示を更新する
    End Sub

    '
    '       ビット３がクリックされたときのイベント処理
    '
    Private Sub PictureBox4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox4.Click
        InvertBitPortB(3)
        UpdateBit(PortBValue, &HFF, 3, PictureBox4, 0)   ' PB3の表示を更新する
    End Sub

    '
    '       ビット４がクリックされたときのイベント処理
    '
    Private Sub PictureBox5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox5.Click
        InvertBitPortB(4)
        UpdateBit(PortBValue, &HFF, 4, PictureBox5, 0)   ' PB4の表示を更新する
    End Sub

    '
    '       ビット５がクリックされたときのイベント処理
    '
    Private Sub PictureBox6_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox6.Click
        InvertBitPortB(5)
        UpdateBit(PortBValue, &HFF, 5, PictureBox6, 0)   ' PB5の表示を更新する
    End Sub

    '
    '       ビット６がクリックされたときのイベント処理
    '
    Private Sub PictureBox7_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox7.Click
        InvertBitPortB(6)
        UpdateBit(PortBValue, &HFF, 6, PictureBox7, 0)   ' PB6の表示を更新する
    End Sub

    '
    '       ビット７がクリックされたときのイベント処理
    '
    Private Sub PictureBox8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox8.Click
        InvertBitPortB(7)
        UpdateBit(PortBValue, &HFF, 7, PictureBox8, 0)   ' PB7の表示を更新する
    End Sub

    '
    '       16進文字を数値に変換する
    '
    Private Shared Function GetHexDigit(ByVal dig As Char) As Byte
        If (Asc(dig) <= Asc("A")) Then
            Return (Asc(dig) And &HF)
        Else
            Return ((Asc(dig) And &HF) + 9)
        End If
    End Function

    '
    '       スレッドの処理
    '
    Private Sub BackgroundWorker1_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundWorker1.DoWork
        Dim newValue As Byte
        Dim recvMessage As String
        While (ThreadContinue)
            If (SerialPort1.IsOpen) Then
                Try
                    recvMessage = SerialPort1.ReadLine()
                    newValue = (GetHexDigit(recvMessage.Substring(2, 1)) << 4) + GetHexDigit(recvMessage.Substring(3, 1))
                    If (recvMessage.Substring(1, 1) = "B") Then
                        Invoke(UpdatePortBDelegate, newValue)
                    ElseIf (recvMessage.Substring(1, 1) = "D") Then
                        Invoke(UpdatePortDDelegate, newValue)
                    End If
                Catch
                End Try
            Else
                Thread.Sleep(5)
            End If
        End While
    End Sub



    '
    '       自動送出パターン
    '
    Dim PortPattern() As Integer = { _
        &H101, &H102, &H104, &H108, &H110, &H120, &H140, &H180, _
        &H101, &H102, &H104, &H108, &H110, &H120, &H140, &H180, _
        &H101, &H102, &H104, &H108, &H110, &H120, &H140, &H180, _
 _
        &H101, &H102, &H104, &H108, &H111, &H122, &H144, &H188, _
        &H111, &H122, &H144, &H188, &H111, &H122, &H144, &H188, _
        &H111, &H122, &H144, &H188, &H111, &H122, &H144, &H188, _
 _
        &H111, &H123, &H146, &H18C, &H118, &H130, &H160, &H1C0, _
        &H181, &H103, &H106, &H10C, &H118, &H130, &H160, &H1C0, _
        &H181, &H103, &H106, &H10C, &H118, &H130, &H160, &H1C0, _
 _
        &H181, &H103, &H106, &H10C, &H118, _
        &H131, &H163, &H1C6, &H18C, &H118, _
        &H131, &H163, &H1C6, &H18C, &H118, _
        &H131, &H163, &H1C6, &H18C, &H118, _
        &H131, &H163, &H1C6, &H18C, &H118, _
 _
        &H131, &H163, _
        &H1C7, &H18E, &H11C, &H138, &H171, &H1E3, _
        &H1C7, &H18E, &H11C, &H138, &H171, &H1E3, _
        &H1C7, &H18E, &H11C, &H138, &H171, &H1E3, _
        &H1C7, &H18E, &H11C, &H138, _
 _
        &H170, &H1E1, &H1C3, &H186, &H10C, &H118, _
        &H130, &H161, &H1C3, &H186, &H10C, &H118, _
        &H130, &H161, &H1C3, &H186, &H10C, &H118, _
        &H130, &H161, &H1C3, &H186, &H10C, &H118, _
        &H130, &H161, &H1C3, &H186, &H10C, &H118, _
        &H130, &H160, &H1C0, &H180, _
 _
        &H101, &H102, &H104, &H108, &H110, &H120, &H140, &H180, _
        &H101, &H102, &H104, &H108, &H110, &H120, &H140, &H180, _
 _
        &H101, _
        &H102, _
        &H104, _
        &H108, _
        &H110, &H110, _
        &H120, &H120, &H120, &H120, _
        &H140, &H140, &H141, &H142, &H144, &H142, &H141, &H140, _
        &H180, &H181, &H182, &H184, &H188, &H190, &H1A0, &H1C0, _
        &H1A0, &H190, &H188, &H184, &H182, &H181, &H182, &H184, _
        &H182, &H181, &H180, &H180 _
    }

    '
    '       パターン自動送出のための変数
    '
    Dim AutoCount As Integer
    Dim AutoIntervalCount As Integer
    Dim Interval As Integer

    '
    '       8ビットのビット並び順を入れ替える
    '
    Protected Function InvertByte(ByVal val As Byte) As Byte
        Dim work As Byte
        For index As Integer = 0 To 7   'ビット数分繰返す
            work = work << 1            '　結果データを1ビット左シフト
            If (val And 1) Then         '　元データの最下位ビットがONなら
                work += 1               '　結果データの最下位ビットをONにする
            End If
            val = val >> 1              '　元データを右シフトする
        Next
        Return work
    End Function

    '
    '       タイマー・イベント処理
    '
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Dim NextValue As Byte
        If (PortDValue And &H40) Then                                   'PD6がONなら、
            Interval = ((PortDValue And &HC) >> 2) + 1                  '間隔を設定する
            If ((AutoIntervalCount < 0) Or (AutoIntervalCount >= Interval)) Then  '自動間隔カウンタがレンジを外れていれば、
                AutoIntervalCount = 0                                        '　自動間隔カウンタをゼロにする
            End If
            AutoIntervalCount += 1                                          '自動間隔カウンタを＋１する
            If (AutoIntervalCount = Interval) Then                          '自動間隔カウンタが間隔を超えていたら
                AutoIntervalCount = 0                                           '自動間隔をゼロにする
                AutoCount += 1                                                  '自動カウンタを＋１する
                If (AutoCount >= PortPattern.Count) Then AutoCount = 0 '自動カウンタがパターンサイズを超えたらゼロにする
                NextValue = PortPattern(AutoCount) And &HFF                     '次のデータをパターンテーブルから取得する
                If (PortDValue And (1 << 5)) Then                               'PD5がONなら
                    NextValue = InvertByte(NextValue)                               'パターンのビット順を反転する
                End If
                If (PortDValue And (1 << 4)) Then                               'PD4がONなら
                    NextValue = Not NextValue                                       'パターンを反転する
                End If
                WritePort("W", "B", NextValue)                                  'データをファームウェアに送出する
                UpdatePortB(NextValue)                                          'ポートBの値を設定し、表示を更新する
            End If
        End If
    End Sub
End Class
