﻿Imports System
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Windows.Forms
Imports System.IO.Ports
Imports System.Text

'==========================================================
'
'       クラス：ポート値表示
'
'==========================================================
Public Class PortView

    Public Const BITOFF As String = "BitOff"        'BitOffイメージ用のキー
    Public Const BITON As String = "BitOn"          'BitOnイメージ用のキー

    '----------------------------------------------------------
    '
    '       ポート値
    '
    '----------------------------------------------------------
    Private m_bitValue As Byte
    Public Overridable Property Value() As Byte
        Get
            Return m_bitValue
        End Get
        Set(ByVal value As Byte)
            m_bitValue = value
            ShowValue()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベル表示イネーブル
    '
    '----------------------------------------------------------
    Private m_labelEnable As Boolean = True
    Public Property LabelEnable() As Boolean
        Get
            Return m_labelEnable
        End Get
        Set(ByVal value As Boolean)
            m_labelEnable = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベル表示位置
    '
    '          True: 上部
    '          False:下部
    '
    '----------------------------------------------------------
    Private m_labelPosition As Boolean = True
    Public Property LabelPosition() As Boolean
        Get
            Return m_labelPosition
        End Get
        Set(ByVal value As Boolean)
            m_labelPosition = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベルギャップ
    '
    '          ラベルとビットパターン間の距離(ピクセル)
    '
    '----------------------------------------------------------
    Private m_labelGap As Integer = 2
    Public Property LabelGap() As Integer
        Get
            Return m_labelGap
        End Get
        Set(ByVal value As Integer)
            m_labelGap = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベル表示色
    '
    '----------------------------------------------------------
    Private m_labelColor As Color = Color.Black
    Public Property LabelColor() As Color
        Get
            Return m_labelColor
        End Get
        Set(ByVal value As Color)
            m_labelColor = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベル背景色
    '
    '----------------------------------------------------------
    Private m_labelBackColor As Color = Color.White
    Public Property LabelBackColor() As Color
        Get
            Return m_labelBackColor
        End Get
        Set(ByVal value As Color)
            m_labelBackColor = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベルの先頭文字列
    '
    '----------------------------------------------------------
    Private m_labelPre As String
    Property LabelPrefix() As String
        Get
            Return m_labelPre
        End Get
        Set(ByVal value As String)
            m_labelPre = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ラベルの後尾文字列
    '
    '----------------------------------------------------------
    Private m_labelPost As String
    Public Property LabelPostfix() As String
        Get
            Return m_labelPost
        End Get
        Set(ByVal value As String)
            m_labelPost = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビットイメージのサイズ
    '
    '----------------------------------------------------------
    Private m_imageSize As Size = New Size(32, 32)
    Property ImageSize() As Size
        Get
            Return m_imageSize
        End Get
        Set(ByVal value As Size)
            m_imageSize = value
            If m_imageSize.Width = 0 Or m_imageSize.Height = 0 Then m_imageSize = New Size(32, 32)
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビットイメージ間の距離(ピクセル）
    '
    '----------------------------------------------------------
    Private m_imageGap As Integer = 10
    Public Property ImageGap() As Integer
        Get
            Return m_imageGap
        End Get
        Set(ByVal value As Integer)
            m_imageGap = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビットパターンのリスト
    '
    '          (0)：デザイン時のフレーム
    '          (1)：ビットOff時のビットパターン
    '          (2)：ビットOn時のビットパターン
    '          インデックスが負のとき、パターンを登録する。
    '          ただし、(0)は初期値登録済み
    '
    '----------------------------------------------------------
    Public ReadOnly Property ImageList(ByVal key As String) As Image
        Get
            If ImageList1.Images.ContainsKey(key) Then
                Return ImageList1.Images(key)
            Else
                Return ImageList1.Images(0)
            End If
        End Get
    End Property

    Public Sub ImageAdd(ByVal key As String, ByVal value As Image)
        ImageList1.Images.Add(key, value)
    End Sub

    '----------------------------------------------------------
    '
    '       イメージリストに登録されているビットパターンの数
    '
    '----------------------------------------------------------
    Public ReadOnly Property ImageListCount() As Integer
        Get
            Return ImageList1.Images.Count
        End Get
    End Property

    '----------------------------------------------------------
    '
    '       ビット列を表示するときの左側オフセット
    '
    '----------------------------------------------------------
    Private m_leftOffset As Integer = 0
    Public Property LeftOffset() As Integer
        Get
            Return m_leftOffset
        End Get
        Set(ByVal value As Integer)
            m_leftOffset = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       コンポーネントの外枠を決定するための右側オフセット
    '
    '----------------------------------------------------------
    Private m_rightOffset As Integer = 0
    Public Property RightOffset() As Integer
        Get
            Return m_rightOffset
        End Get
        Set(ByVal value As Integer)
            m_rightOffset = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       内部変数
    '          ラベル参照変数配列
    '          ビット表示参照変数配列
    '
    '----------------------------------------------------------
    Protected m_bitLabel(8) As Label
    Protected m_bitImage(8) As PictureBox

    '----------------------------------------------------------
    '
    '       コンストラクタ
    '
    '----------------------------------------------------------
    Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。
        m_bitLabel(0) = Label1
        m_bitLabel(1) = Label2
        m_bitLabel(2) = Label3
        m_bitLabel(3) = Label4
        m_bitLabel(4) = Label5
        m_bitLabel(5) = Label6
        m_bitLabel(6) = Label7
        m_bitLabel(7) = Label8
        m_bitImage(0) = PictureBox1
        m_bitImage(1) = PictureBox2
        m_bitImage(2) = PictureBox3
        m_bitImage(3) = PictureBox4
        m_bitImage(4) = PictureBox5
        m_bitImage(5) = PictureBox6
        m_bitImage(6) = PictureBox7
        m_bitImage(7) = PictureBox8
        For index As Integer = 0 To 7
            'm_bitLabel(index) = New Label                                   'Labelを生成する
            'm_bitLabel(index).TextAlign = ContentAlignment.MiddleCenter     '文字列を縦中央、横中央に配置す
            'Controls.Add(m_bitLabel(index))                                 '親にコントロールを登録する

            'm_bitImage(index) = New PictureBox                              'PictureBoxを生成する
            'm_bitImage(index).SizeMode = PictureBoxSizeMode.StretchImage    'サイズが異なるとき、イメージをストレッチする
            'm_bitImage(index).Tag = index                                   'タグにビット番号を設定する
            'm_bitImage(index).Image = ImageList1.Images.Item(0)             '設計モード用パターンの設定
            'Controls.Add(m_bitImage(index))                                 '親にコントロールを登録する
            'AddHandler m_bitImage(index).Click, AddressOf PictureBox_Click  'PictureBoxクリック時のイベントハンドラーを設定する
        Next
    End Sub

    '----------------------------------------------------------
    '
    '       フォーム読込み時の初期設定
    '
    '----------------------------------------------------------
    Private Sub PortView_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LayoutComponent()                   '部品内のレイアウトを整える
    End Sub

    '----------------------------------------------------------
    '
    '       部品内レイアウトの計算
    '
    '----------------------------------------------------------
    Protected Overridable Sub LayoutComponent()
        Dim labelTop As Integer
        Dim imageTop As Integer
        Const labelHeight = 14
        'ラベルとイメージの表示位置（TOP位置）を設定する
        If m_labelPosition Then
            labelTop = 2
            If m_labelEnable Then
                imageTop = labelHeight + LabelGap + 2
            Else
                imageTop = 2
            End If
        Else
            imageTop = 2
            labelTop = ImageSize.Height + 2 + LabelGap
        End If

        For index As Integer = 0 To 7
            m_bitLabel(index).Visible = m_labelEnable       'ラベルの表示／非表示を設定
            If m_labelEnable Then                           'ラベル表示なら、ラベルのサイズと位置を算出
                m_bitLabel(index).Size = New Size(m_imageSize.Width, labelHeight)
                m_bitLabel(index).Location = New Point((7 - index) * (ImageSize.Width + ImageGap) + ImageGap / 2 + m_leftOffset, labelTop)
            End If

            m_bitImage(index).Size = ImageSize              'イメージサイズを設定する
            m_bitImage(index).Location = New Point((7 - index) * (ImageSize.Width + ImageGap) + ImageGap / 2 + m_leftOffset, imageTop)  'イメージの表示位置を算出
            If m_labelEnable Then                           'ラベル表示なら、
                m_bitLabel(index).Text = String.Format("{0}{1}{2}", LabelPrefix, index, LabelPostfix)       'ラベルに文字列を設定
                m_bitLabel(index).ForeColor = LabelColor        'ラベルの色を設定
                m_bitLabel(index).BackColor = LabelBackColor    'ラベルの背景色を設定
            End If
            ShowBit(index)                                      '当該ビットの画像を表示
        Next
        Width = 8 * ImageSize.Width + 9 * ImageGap + m_leftOffset + m_rightOffset     'コントロール全体の幅を算出
        If m_labelEnable Then                                                       'ラベル表示なら、
            Height = ImageSize.Height + LabelGap + labelHeight + 8                      'コントロールの高さをラベル込で算出
        Else                                                                        'ラベル非表示なら
            Height = ImageSize.Height + LabelGap * 2 + 4                                    'コントロールの高さをラベル抜きで算出
        End If
    End Sub

    '----------------------------------------------------------
    '
    '       指定したビットを反転する
    '
    '----------------------------------------------------------
    Public Sub NotBit(ByVal bitPos As Integer)
        Value = Value Xor (1 << bitPos)
    End Sub

    '----------------------------------------------------------
    '
    '       指定したビットをセットする
    '
    '----------------------------------------------------------
    Public Sub SetBit(ByVal bitPos As Integer)
        Value = Value Or (1 << bitPos)
    End Sub

    '----------------------------------------------------------
    '
    '       指定したビットをリセットする
    '
    '----------------------------------------------------------
    Public Sub ResetBit(ByVal bitPos As Integer)
        Value = Value And (Not (1 << bitPos))
    End Sub

    '----------------------------------------------------------
    '
    '       指定したビットのビット・イメージを更新する
    '
    '----------------------------------------------------------
    Public Overridable Sub ShowBit(ByVal bitPos As Integer)
        If (Value And (1 << bitPos)) <> 0 Then                      '当該ビットがONなら
            If ImageList1.Images.ContainsKey(BITON) Then
                m_bitImage(bitPos).Image = ImageList(BITON)             'ONの画像を設定
            End If
        Else                                                        '　OFFなら
            If ImageList1.Images.ContainsKey(BITOFF) Then
                m_bitImage(bitPos).Image = ImageList(BITOFF)            'OFF画像を設定
            End If
        End If
    End Sub

    '----------------------------------------------------------
    '
    '       データを表示する
    '
    '----------------------------------------------------------
    Public Sub ShowValue()
        For index As Integer = 0 To 7
            ShowBit(index)
        Next
    End Sub

    '----------------------------------------------------------
    '
    '       ビットがクリックされたときのイベント定義
    '
    '----------------------------------------------------------
    Public Event PortValueChanged(ByVal sender As PortView, ByVal bitPosition As Integer)

    '----------------------------------------------------------
    '
    '       ビットがクリックされたら、イベントを発生させる
    '
    '----------------------------------------------------------
    Private Sub PictureBox_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PictureBox8.Click, PictureBox7.Click, PictureBox6.Click, PictureBox5.Click, PictureBox4.Click, PictureBox3.Click, PictureBox2.Click, PictureBox1.Click
        RaiseEvent PortValueChanged(Me, CType(CType(sender, PictureBox).Tag, Integer))
    End Sub

End Class


'==========================================================
'
'       クラス：ビット有効/無効つき、ポート値表示
'
'==========================================================
Public Class PortViewWithEnable : Inherits PortView

    '----------------------------------------------------------
    '
    '       ビットパターン（NotAvailable）の格納位置
    '
    '----------------------------------------------------------
    Public Const SWITCHOFF As String = "SwitchOff"
    Public Const SWITCHON As String = "SwitchOn"
    Public Const SWITCHNA As String = "SwitchNA"

    '----------------------------------------------------------
    '
    '       ビット表示
    '
    '----------------------------------------------------------
    Private m_bitDisplay As Byte = &HFF
    Public Property BitDisplay() As Byte
        Get
            Return m_bitDisplay
        End Get
        Set(ByVal value As Byte)
            m_bitDisplay = value
            ShowValue()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビット有効/無効
    '
    '----------------------------------------------------------
    Private m_bitEnable As Byte = &HFF
    Public Property BitEnable() As Byte
        Get
            Return m_bitEnable
        End Get
        Set(ByVal value As Byte)
            m_bitEnable = value
            ShowValue()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビットパターン表示
    '
    '----------------------------------------------------------
    Public Overrides Sub ShowBit(ByVal bitPos As Integer)
        If ((BitEnable And BitDisplay) And (1 << bitPos)) <> 0 Then     '当該ビットを表示するか？
            If (Value And (1 << bitPos)) <> 0 Then
                If ImageList1.Images.ContainsKey(SWITCHON) Then
                    m_bitImage(bitPos).Image = ImageList(SWITCHON)          '当該ビットはON
                End If
            Else
                If ImageList1.Images.ContainsKey(SWITCHOFF) Then
                    m_bitImage(bitPos).Image = ImageList(SWITCHOFF)         '当該ビットはOFF
                End If
            End If
        Else
            If ImageList1.Images.ContainsKey(SWITCHNA) Then
                m_bitImage(bitPos).Image = ImageList(SWITCHNA)          '当該ビットはNotAvailableとする
            End If
        End If

        If (BitDisplay And (1 << bitPos)) <> 0 Then                     'BitDisplayがONなら
            m_bitImage(bitPos).Visible = True                               '当該ビットを表示
            m_bitLabel(bitPos).Visible = True
        Else                                                            'OFFなら
            m_bitImage(bitPos).Visible = False                              '当該ビットを非表示
            m_bitLabel(bitPos).Visible = False
        End If
    End Sub

End Class


'==========================================================
'
'       クラス：バイトデータ編集
'
'==========================================================
Public Class PortViewEdit : Inherits PortView

    '----------------------------------------------------------
    '
    '       ポート値
    '
    '----------------------------------------------------------
    Public Overrides Property Value() As Byte
        Get
            Return MyBase.Value
        End Get
        Set(ByVal value As Byte)
            m_valueDspDLabel.Text = value                                       '10進表示
            m_valueDspHLabel.Text = String.Format("(&&H{0:X2})", value)         '16進表示
            MyBase.Value = value                                                '親クラスのプロパティ処理
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       バイト位置
    '
    '----------------------------------------------------------
    Private m_valuePosition As Integer
    Public Property ValuePosition() As Integer
        Get
            Return m_valuePosition
        End Get
        Set(ByVal value As Integer)
            m_valuePosition = value                                             '値をセット
            m_valuePosDLabel.Text = value                                       '10進表示
            m_valuePosHLabel.Text = String.Format("(&&H{0:X4})", value)         '16進表示
            LayoutComponent()
        End Set
    End Property


    '----------------------------------------------------------
    '
    '       バイト位置とバイト値の表示ラベル
    '
    '----------------------------------------------------------
    Private m_valuePosDLabel As Label
    Private m_valuePosHLabel As Label
    Private m_valueDspDLabel As Label
    Private m_valueDspHLabel As Label

    '----------------------------------------------------------
    '
    '       コンストラクタ
    '
    '----------------------------------------------------------
    Sub New()
        MyBase.New()
        m_valuePosDLabel = New Label
        m_valuePosDLabel.TextAlign = ContentAlignment.MiddleRight
        Controls.Add(m_valuePosDLabel)
        m_valuePosHLabel = New Label
        m_valuePosHLabel.TextAlign = ContentAlignment.MiddleLeft
        Controls.Add(m_valuePosHLabel)
        m_valueDspDLabel = New Label
        m_valueDspDLabel.TextAlign = ContentAlignment.MiddleRight
        Controls.Add(m_valueDspDLabel)
        m_valueDspHLabel = New Label
        m_valueDspHLabel.TextAlign = ContentAlignment.MiddleLeft
        Controls.Add(m_valueDspHLabel)
        LeftOffset = 130
        RightOffset = 100
        Width += 230
    End Sub

    '----------------------------------------------------------
    '
    '       画面レイアウトの計算、配置
    '
    '----------------------------------------------------------
    Protected Overrides Sub LayoutComponent()
        MyBase.LayoutComponent()
        m_valuePosDLabel.Location = New Point(2, m_bitImage(0).Top)
        m_valuePosDLabel.Size = New Size(50, m_bitImage(0).Height)
        m_valuePosHLabel.Location = New Point(55, m_bitImage(0).Top)
        m_valuePosHLabel.Size = New Size(70, m_bitImage(0).Height)

        m_valueDspDLabel.Location = New Point(m_bitImage(0).Left + m_bitImage(0).Width + 5, m_bitImage(0).Top)
        m_valueDspDLabel.Size = New Size(35, m_bitImage(0).Height)
        m_valueDspHLabel.Location = New Point(m_bitImage(0).Left + m_bitImage(0).Width + 45, m_bitImage(0).Top)
        m_valueDspHLabel.Size = New Size(55, m_bitImage(0).Height)
    End Sub

End Class


