﻿
Imports System.Windows.Forms
Imports System.IO

Public Class PatternEdit

    '----------------------------------------------------------
    '
    '       作業用パターンテーブル
    '
    '----------------------------------------------------------
    Public PortPatternSize As Integer = 0
    Public PortPattern(AutoPattern.PATTERNBUFFSIZE) As Byte

    '----------------------------------------------------------
    '
    '       イメージリスト
    '
    '----------------------------------------------------------
    Public ReadOnly Property Images(ByVal pos As Integer) As Image
        Get
            Return patEdit(0).ImageList(pos)
        End Get
    End Property

    Public Sub ImageAdd(ByVal key As String, ByVal value As Image)
        For index As Integer = 0 To pagePerByte - 1
            patEdit(index).ImageAdd(key, value)
        Next
    End Sub

    '----------------------------------------------------------
    '
    '       １画面に表示するバイト数
    '
    '----------------------------------------------------------
    Dim pagePerByte As Integer = 38

    '----------------------------------------------------------
    '
    '       当該画面先頭データのパターンテーブル上の位置
    '
    '----------------------------------------------------------
    Dim positionIndex As Integer = 0
    Private patEdit(pagePerByte) As PortView.PortViewEdit

    Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。

        For index As Integer = 0 To pagePerByte - 1
            patEdit(index) = New PortView.PortViewEdit                          'バイト編集コントロールの生成
            patEdit(index).Tag = index                                          'Tagにビット位置を設定
            patEdit(index).LabelEnable = False                                  'ラベル表示を禁止する
            patEdit(index).ImageSize = New Size(12, 12)                         'イメージサイズを設定する
            patEdit(index).ImageGap = 4                                         'ビット間を４ドットにする
            patEdit(index).Location = New Point(2, index * (patEdit(index).Height - 2))     '表示位置を設定する
            EditPanel.Controls.Add(patEdit(index))                              'コントロールを親コントロールに登録する
            AddHandler patEdit(index).PortValueChanged, AddressOf PortViewEdit_PortValueChanged     'マウスくりっきじのイベントを登録する
        Next
        VScrollBar1.LargeChange = pagePerByte                                   'スクロールバーのページ送りサイズを設定
        VScrollBar1.Maximum = AutoPattern.PATTERNBUFFSIZE                       'スクロールバーの最大値を設定
        DataSize.Maximum = AutoPattern.PATTERNBUFFSIZE                          'データサイズ（数値上下コントロール）を設定
    End Sub

    '----------------------------------------------------------
    '
    '       ページ↑スクロール
    '
    '----------------------------------------------------------
    Private Sub PageUp()
        positionIndex = positionIndex + 5                                       'ページ送り量だけ、表示位置を更新する
        If positionIndex >= (AutoPattern.PATTERNBUFFSIZE - pagePerByte) Then    '表示がページ内に収まるよう、
            positionIndex = AutoPattern.PATTERNBUFFSIZE - pagePerByte               '表示位置を調整
        End If
        VScrollBar1.Value = positionIndex                                       'スクロールバーのスクロール位置を設定
    End Sub

    '----------------------------------------------------------
    '
    '       ページ↓スクロール
    '
    '----------------------------------------------------------
    Private Sub PageDown()
        positionIndex = positionIndex - 5                                       'ページ送り量だけ、表示位置を更新する
        If (positionIndex < 0) Then positionIndex = 0
        VScrollBar1.Value = positionIndex                                       'スクロールバーのスクロール位置を設定
    End Sub

    '----------------------------------------------------------
    '
    '       ページ表示を更新する
    '
    '----------------------------------------------------------
    Private Sub UpdatePage()
        For index As Integer = 0 To pagePerByte - 1
            If (positionIndex + index) < AutoPattern.PATTERNBUFFSIZE Then       'テーブル範囲内のデータであれば
                patEdit(index).ValuePosition = positionIndex + index                'バイト位置を設定する
                patEdit(index).Value = PortPattern(positionIndex + index)           'バイト値を設定する
            End If
        Next
    End Sub


    '----------------------------------------------------------
    '
    '       OKボタン（終了）処理
    '
    '----------------------------------------------------------
    Private Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.OK                  '正常終了を設定して
        Me.Close()                                                              'ダイアログを閉じる
    End Sub

    '----------------------------------------------------------
    '
    '       Canselボタン（取り消し）処理
    '
    '----------------------------------------------------------
    Private Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel              'キャンセル終了を設定して
        Me.Close()                                                              'ダイアログを閉じる
    End Sub

    '----------------------------------------------------------
    '
    '       マウスホイール処理
    '
    '----------------------------------------------------------
    Private Sub PatternEdit_MouseWheel(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseWheel ', EditPanel.MouseWheel, VScrollBar1.MouseWheel
        Dim w As Integer
        w = e.Delta                                                             'マウスの回転を取得
        If w > 0 Then                                                           '後ろに回転時、
            PageDown()                                                              'ページ↓スクロール
        Else                                                                    '前に回転時
            PageUp()                                                                'ページ↑スクロール
        End If
        UpdatePage()                                                            'ページ表示を更新する
    End Sub

    '----------------------------------------------------------
    '
    '       ウィンドウ表示
    '
    '----------------------------------------------------------
    Private Sub PatternEdit_Shown(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Shown
        DataSize.Value = PortPatternSize
        UpdatePage()                                                            'ページ表示を更新する
    End Sub

    '----------------------------------------------------------
    '
    '       データクリック時の処理
    '
    '----------------------------------------------------------
    Private Sub PortViewEdit_PortValueChanged(ByVal sender As PortView.PortView, ByVal bitPosition As System.Int32)
        Dim bytePos As Integer = CType(CType(sender, PortView.PortViewEdit).Tag, Integer)   'SendarのTagからバイト位置を取得する
        patEdit(bytePos).NotBit(bitPosition)                                        '当該バイト・ビットを反転する
        PortPattern(positionIndex + bytePos) = patEdit(bytePos).Value               'パターンテーブルのデータを更新する
    End Sub

    '----------------------------------------------------------
    '
    '       スクロールバー操作時の処理
    '
    '----------------------------------------------------------
    Private Sub VScrollBar1_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles VScrollBar1.ValueChanged
        positionIndex = CType(sender, ScrollBar).Value                              '新しいスクロール位置を取得する
        If positionIndex >= (AutoPattern.PATTERNBUFFSIZE - pagePerByte) Then        '表示がページ内に収まるよう、
            positionIndex = AutoPattern.PATTERNBUFFSIZE - pagePerByte                   '表示位置を調整
        End If
        UpdatePage()                                                                'ページ表示を更新する
    End Sub

    '----------------------------------------------------------
    '
    '       データサイズの変更
    '
    '----------------------------------------------------------
    Private Sub DataSize_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataSize.ValueChanged
        PortPatternSize = DataSize.Value                                        'データサイズを更新する
    End Sub

    '----------------------------------------------------------
    '
    '       データクリアー
    '
    '----------------------------------------------------------
    Private Sub ClearButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ClearButton.Click
        For index As Integer = 0 To AutoPattern.PATTERNBUFFSIZE
            PortPattern(index) = 0
        Next

        DataSize.Value = 0                                                      'データサイズをゼロにする
        positionIndex = 0                                                       '表示開始位置をゼロにする
        VScrollBar1.Value = positionIndex                                       'スクロールバーを設定する
        UpdatePage()                                                            'ページ表示を更新する
    End Sub

    '----------------------------------------------------------
    '
    '       １６進文字列を数値に変換
    '
    '----------------------------------------------------------
    Private Function GetHexDigit(ByVal dig As Byte) As Byte
        If (dig > &H39) Then                                                    '数字の９より大きいとき、
            Return (dig And &HF) + 9                                                '下位４ビットだけ取出して、９を足す
        Else                                                                    '９以下のとき
            Return (dig And &HF)                                                    '下位４ビットだけ取り出す
        End If
    End Function

    '----------------------------------------------------------
    '
    '       １６進ファイル読込み
    '
    '----------------------------------------------------------
    Private Sub ReadHex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReadHex.Click
        Dim rfile As StreamReader               'テキストファイルの読込み用ファイル
        Dim rfileLength As Integer = 0          'データ長
        Dim rchar As Byte                       '読込みデータ
        OpenFileDialog1.Filter = "Text File|*.txt|All Files|*.*"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then    'ダイアログでファイル名を取得
            Try
                rfile = File.OpenText(OpenFileDialog1.FileName)                 'ファイルを読込み用にオープンする
                Do While rfile.Peek >= 0                                        'データがあれば繰返す
                    rchar = rfile.Read()                                            'データを読み込む
                    While (rchar <> &H26)                                           '文字が＆で無ければ、繰返す
                        If rfile.Peek < 0 Then Exit While '                             次のデータが無いとき終了する
                        rchar = rfile.Read()                                            '次のデータを読み込む
                    End While
                    If rfile.Peek < 0 Then Exit Do '                                 次のデータが無ければ終了する
                    rchar = rfile.Read()                                            '次のデータを読み込む
                    If rchar = &H48 Then                                            '次のデータは"H"か？
                        If rfile.Peek < 0 Then Exit Do '                                 次のデータが無ければ終了する
                        PortPattern(rfileLength) = (GetHexDigit(rfile.Read()) << 4)     'データの上位４ビットを取得
                        If rfile.Peek < 0 Then Exit Do '                                 次のデータが無ければ終了する
                        PortPattern(rfileLength) += GetHexDigit(rfile.Read())           'データの下位４ビットを取得
                        rfileLength += 1                                                '読込みデータ長を更新
                    End If
                    If rfileLength >= AutoPattern.PATTERNBUFFSIZE Then Exit Do '読込みデータ長がバッファサイズを超えたら終了
                Loop
                DataSize.Value = rfileLength                                    'データサイズを設定する
                positionIndex = 0                                               '表示開始位置をゼロにする
                VScrollBar1.Value = positionIndex                               'スクロールバーを設定する
                UpdatePage()                                                    '表示を更新する
            Catch
                MessageBox.Show("何か、エラーが起きました。")
            End Try
        End If
    End Sub

    '----------------------------------------------------------
    '
    '       バイナリファイル読込み
    '
    '----------------------------------------------------------
    Private Sub ReadBinary_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReadBinary.Click
        Dim rfile As FileStream                                     'バイナリファイルの読込用
        Dim rfileLength As Integer                                  'レコード長
        OpenFileDialog1.Filter = "Binary Data File (*.dat)|*.dat|All Files (*.*)|*.*"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then        'ダイアログでファイル名取得
            Try
                rfile = File.Open(OpenFileDialog1.FileName, FileMode.Open, FileAccess.Read)         'ファイルを開く
                rfileLength = rfile.Length                                                          'ファイル長を取得
                If rfileLength > AutoPattern.PATTERNBUFFSIZE Then rfileLength = AutoPattern.PATTERNBUFFSIZE 'ファイル長の制限
                For index As Integer = 0 To rfileLength - 1                         '読込サイズ分繰返す
                    PortPattern(index) = rfile.ReadByte()                           'データを１バイト読み込んでパターンテーブルに格納
                Next
                PortPatternSize = rfileLength                                       'データ長を更新
                rfile.Close()                                                       'ファイルを閉じる

                DataSize.Value = rfileLength                                        'データ長を設定
                positionIndex = 0                                                   '表示開始位置を設定
                VScrollBar1.Value = positionIndex                                   'スクロールバーを設定
                UpdatePage()                                                        '表示を更新する
            Catch
                MessageBox.Show("何か、エラーが起きました。")
            End Try
        End If
    End Sub

    '----------------------------------------------------------
    '
    '       １６進ファイル書き込み
    '
    '----------------------------------------------------------
    Private Sub WriteHex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles WriteHex.Click
        Dim wfile As StreamWriter                                   'テキストファイル書込み用
        If PortPatternSize > 0 Then                                 'データサイズは１以上か？
            SaveFileDialog1.Filter = "Text File (*.txt)|*.txt|All Files (*.*)|*.*"
            If SaveFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then    'ダイアログで出力ファイル名を取得
                wfile = File.CreateText(SaveFileDialog1.FileName)                   'ファイル名をセットして、ファイルを作成する
                wfile.Write("{ " & String.Format("&H{0:X2}", PortPattern(0)))       '最初の１バイトを出力。先頭に"{"をつける。データの書式は&Hxx
                For index As Integer = 1 To PortPatternSize - 1                     'バイト数分繰返す
                    wfile.Write(String.Format(", &H{0:X2}", PortPattern(index)))        '１バイト出力。データの書式は&Hxx
                    If ((index + 1) Mod 16) = 0 Then                                    '１６バイトごとに
                        wfile.WriteLine(" _")                                           'VBの行継続文字を入れて改行
                    End If
                Next
                wfile.WriteLine(" }")                                               '最後に"}"をつける
                wfile.Flush()                                                       'バッファをフラッシュ（ファイルに吐き出す）して
                wfile.Close()                                                       'ファイルを閉じる
            End If
        Else
            MessageBox.Show("出力するデータがありません。")
        End If
    End Sub

    '----------------------------------------------------------
    '
    '       バイナリー・ファイル書き込み
    '
    '----------------------------------------------------------
    Private Sub WriteBinary_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles WriteBinary.Click
        Dim wfile As FileStream                                         'バイナリ出力用
        If PortPatternSize > 0 Then                                     'データ数が１以上か？
            SaveFileDialog1.Filter = "Binary Data File (*.dat)|*.dat|All Files (*.*)|*.*"
            If SaveFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then    '出力ファイル名を取得する
                wfile = File.Create(SaveFileDialog1.FileName, 2048)                     'ファイルを生成する。バッファサイズは２０４８バイト
                For index As Integer = 0 To PortPatternSize - 1                         'データ数分繰返す
                    wfile.WriteByte(PortPattern(index))                                     'パターンデータを１バイト出力
                Next
                wfile.Flush()                                                           'バッファをフラッシュ（ファイルに吐き出す）して
                wfile.Close()                                                           'ファイルを閉じる
            End If
        Else
            MessageBox.Show("出力するデータがありません。")
        End If
    End Sub


End Class
